"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BullDataSource = void 0;
const apollo_datasource_1 = require("apollo-datasource");
const json_1 = require("../../services/json");
const gql_1 = require("../../typings/gql");
const redis_info_1 = __importDefault(require("redis-info"));
class BullDataSource extends apollo_datasource_1.DataSource {
    constructor(_queues) {
        super();
        this._queues = _queues;
        this._queuesMap = new Map();
        this._convertQueuesToMap(_queues);
    }
    _convertQueuesToMap(queues) {
        queues.forEach(queue => {
            this._queuesMap.set(queue.name, queue);
        });
    }
    getQueueByName(name) {
        return this._queuesMap.get(name);
    }
    getQueues() {
        return this._queues;
    }
    async createJob({ queue: queueName, name = null, data = {}, options = {}, }) {
        const queue = this._queuesMap.get(queueName);
        return await (queue === null || queue === void 0 ? void 0 : queue.add(name, json_1.JsonService.maybeParse(data), json_1.JsonService.maybeParse(options)));
    }
    async getQueueJobs({ queue, limit = 20, offset = 0, status = [], id, order = gql_1.OrderEnum.Desc, }) {
        const bullQueue = this.getQueueByName(queue);
        if (id) {
            const job = await (bullQueue === null || bullQueue === void 0 ? void 0 : bullQueue.getJob(id));
            return job ? [job] : [];
        }
        return bullQueue === null || bullQueue === void 0 ? void 0 : bullQueue.getJobs(status, offset, offset + limit - 1, order === gql_1.OrderEnum.Asc);
    }
    async getJob(queueName, id) {
        const queue = this.getQueueByName(queueName);
        return queue === null || queue === void 0 ? void 0 : queue.getJob(id);
    }
    async getJobLogs(queueName, id) {
        const queue = this.getQueueByName(queueName);
        return await (queue === null || queue === void 0 ? void 0 : queue.getJobLogs(id));
    }
    async createJobLog(args) {
        const job = await this.getJob(args.queue, args.id);
        await (job === null || job === void 0 ? void 0 : job.log(args.row));
        return job;
    }
    async retryJob(args) {
        const job = await this.getJob(args.queue, args.id);
        await (job === null || job === void 0 ? void 0 : job.retry());
        return job;
    }
    async removeJobById(args) {
        const job = await this.getJob(args.queue, args.id);
        await (job === null || job === void 0 ? void 0 : job.remove());
        return job;
    }
    async moveJobToCompleted(args) {
        const job = await this.getJob(args.queue, args.id);
        await (job === null || job === void 0 ? void 0 : job.moveToCompleted());
        return job;
    }
    async moveJobToFailed(args) {
        const job = await this.getJob(args.queue, args.id);
        await (job === null || job === void 0 ? void 0 : job.moveToFailed({
            message: '',
        }));
        return job;
    }
    async promoteJob(args) {
        const job = await this.getJob(args.queue, args.id);
        await (job === null || job === void 0 ? void 0 : job.promote());
        return job;
    }
    async discardJob(args) {
        const job = await this.getJob(args.queue, args.id);
        await (job === null || job === void 0 ? void 0 : job.discard());
        return job;
    }
    async updateJobData(args) {
        const job = await this.getJob(args.queue, args.id);
        await (job === null || job === void 0 ? void 0 : job.update(json_1.JsonService.maybeParse(args.data)));
        return job;
    }
    async getQueueJobsCounts(name) {
        const queue = this.getQueueByName(name);
        return await (queue === null || queue === void 0 ? void 0 : queue.getJobCounts());
    }
    async pauseQueue(name) {
        const queue = this.getQueueByName(name);
        await (queue === null || queue === void 0 ? void 0 : queue.pause());
        return queue;
    }
    async cleanQueue(args) {
        const queue = this.getQueueByName(args.queue);
        return await (queue === null || queue === void 0 ? void 0 : queue.clean(args.grace, args.status));
    }
    async emptyQueue(args) {
        const queue = this.getQueueByName(args.queue);
        await (queue === null || queue === void 0 ? void 0 : queue.empty());
        return queue;
    }
    async closeQueue(args) {
        const queue = this.getQueueByName(args.queue);
        await (queue === null || queue === void 0 ? void 0 : queue.close());
        return queue;
    }
    async resumeQueue(args) {
        const queue = this.getQueueByName(args.queue);
        await (queue === null || queue === void 0 ? void 0 : queue.resume());
        return queue;
    }
    async getQueueWaitingOrDelayedJobsCount(name) {
        const queue = this.getQueueByName(name);
        return await (queue === null || queue === void 0 ? void 0 : queue.count());
    }
    async removeJobsByPattern(args) {
        const queue = this.getQueueByName(args.queue);
        await (queue === null || queue === void 0 ? void 0 : queue.removeJobs(args.pattern));
        return true;
    }
    async getRedisInfo() {
        if (this._queuesMap.size > 0) {
            const firstQueue = this._queues[0];
            const rawInfo = await firstQueue.client.info();
            return redis_info_1.default.parse(rawInfo);
        }
        return null;
    }
}
exports.BullDataSource = BullDataSource;
//# sourceMappingURL=bull.js.map